<?php
/**
 * Anwiki is a multilingual content management system <http://www.anwiki.com>
 * Copyright (C) 2007-2008 Antoine Walter <http://www.anw.fr>
 * 
 * Anwiki is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 * 
 * Anwiki is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Anwiki.  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * XML Content, related to a ContentClass or a ContentField.
 * @package Anwiki
 * @version $Id: class_content.php,v 1.22 2008-09-11 22:42:02 antoine-cvs Exp $
 * @copyright 2007-2008 Antoine Walter
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License 3
 */

interface AnwIStructuredContent
{
	static function rebuildContentFromXml($oContentFieldsContainer, $sXmlValue);
}

abstract class AnwStructuredContent implements AnwPubcallCapability, AnwIStructuredContent
{
	protected $oContentFieldsContainer;
	protected $aasContentFieldsValues = array();
	private $oCachedXml;
	private $sCachedXmlString;
	private $bHtmlEditFormHasErroneousFields;
	private $nHtmlEditFormCountErroneousChild;
	private $bHtmlEditFormHasPhpCode;
	private $bHtmlEditFormHasJsCode;
	
	const XMLNODE_VALUE="anwv";
	
	const IDX_SUBCONTENT = 'SUBCONTENT';
	const IDX_RENDERED = 'RENDERED';
	
	function __construct($oContentFieldsContainer)
	{
		$this->oContentFieldsContainer = $oContentFieldsContainer;
	}
	
	abstract function newContent($oContentFieldsContainer);
	
	function getSubContents($sFieldName, $bWithDefaultValues=true)
	{
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		$aoFieldValues = $this->getContentFieldValues($sFieldName, false, $bWithDefaultValues);
		
		//in some case (settings), we have single composed fields with no values, such as <fieldname/>
		//but we have a default value. If bWithDefaultValues, we append the default value. 
		if (!$oContentField->isMultiple() && count($aoFieldValues)==0 && $bWithDefaultValues)
		{
			$aoFieldValues[] = $this->newContent($oContentField)->toXmlString();
		}
		
		$aoSubContents = array();
		foreach ($aoFieldValues as $sFieldValue)
		{
			$aoSubContents[] = $oContentField->rebuildContentFromXml($sFieldValue);
		}
		return $aoSubContents;
	}
	
	function getSubContent($sFieldName)
	{
		$aoSubContents = $this->getSubContents($sFieldName);
		if (!isset( $aoSubContents[0])) throw new AnwUnexpectedException();
		$oSubContent = $aoSubContents[0];
		return $oSubContent;
	}
	
	/*
	protected function completeValueWithDefault($sValueXmlString)
	{
		$oSubContent = AnwContentSettings::rebuildContentFromXml($this->getContentFieldsContainer(), $sValueXmlString);
		
	}*/
	
	function getContentFieldValues($sFieldName, $bStripUntr=false, $bWithDefaultValues=true)
	{
		//check that contentfield exists (or throw an exception)
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		
		//return contentfield's values
		if (isset($this->aasContentFieldsValues[$sFieldName]))
		{
			$asReturn = $this->aasContentFieldsValues[$sFieldName];
			/*if ($bWithDefaultValues)
			{
				foreach ($asReturn as $i => $sValueReturn)
				{
					$asReturn[$i] = $this->completeValueWithDefault($sValueReturn);
				}
			}*/
			if ($bStripUntr)
			{
				foreach ($asReturn as $i=>$null)
				{
					$asReturn[$i] = AnwUtils::stripUntr($asReturn[$i]);
				}
			}
		}
		else
		{
			if ($bWithDefaultValues && $oContentField->hasDefaultValues() /*&& $oContentField instanceof AnwStructuredContentField_atomic && $oContentField->hasDefaultValue()*/)
			{
				AnwDebug::log("WARNING: returning default value for getContentFieldValues(".$sFieldName.")");
				//$asReturn = array($oContentField->getDefaultValue());
				$asReturn = $oContentField->getDefaultValues();
			}
			else
			{
				AnwDebug::log("WARNING: returning NO default value for getContentFieldValues(".$sFieldName.")");
				$asReturn = array();
			}
		}
		return $asReturn;
	}
	
	function getContentFieldValue($sFieldName, $i=0, $bStripUntr=false)
	{
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		if ($oContentField->isMultiple())
		{
			throw new AnwUnexpectedException("getContentFieldValue called on multiple field");
		}
		$asValues = $this->getContentFieldValues($sFieldName, $bStripUntr);
		if (isset($asValues[$i]))
		{
			$sValue = $asValues[$i];
		}
		else
		{
			AnwDebug::log("WARNING: returning default value for getContentFieldValue(".$sFieldName.",".$i.")");
			$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
			$sValue = $oContentField->getDefaultValue();
		}
		return $sValue;
	}
	
	function getContentFieldOutput($sFieldName)
	{
		$sValue = $this->getContentFieldValue($sFieldName);
		
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		return $oContentField->output($sValue);
	}
	
	function getContentFieldValuesAsXml($sFieldName)
	{
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		$asFieldValues = $this->getContentFieldValues($sFieldName);
		
		//create XML
		$oDoc = AnwUtils::newDomDocument();
		$oNodeRoot = $oDoc->createElement('values');
		$oDoc->appendChild($oNodeRoot);
		
		//export field values
		foreach ($asFieldValues as $sFieldValue) //!!Use foreach instead of while which skips empty strings
		{
			$oValueNode = AnwXml::xmlCreateElementWithChilds($oNodeRoot->ownerDocument, self::XMLNODE_VALUE, $sFieldValue);
			$oNodeRoot->appendChild($oValueNode);
		}
		return $oNodeRoot;
	}
	
	function setContentFieldValues($sFieldName, $asFieldValues)
	{
		//check that contentfield exists or throw an exception
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		
		//test values: this is commented for performances issues.
		//in normal situation, we shouldn't need it as testContentFieldValues() should be
		//explicitely called each time we need it.
		//$oContentField->testContentFieldValues($asFieldValues);
		
		$this->aasContentFieldsValues[$sFieldName] = $asFieldValues;
		$this->setXmlChanged();
	}
	
	function setContentFieldValuesFromXml($sFieldName, $oDoc)
	{
		$oRootNode = $oDoc;//$oDoc->documentElement;
		$asValues = array();
		
		//check that contentfield exists or throw an exception
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		
		//loop for each <anwvalue> tag
		$aoValueElements = AnwXml::xmlGetChildsByTagName(self::XMLNODE_VALUE, $oRootNode);
		foreach ($aoValueElements as $oValueElement)
		{
				$mFieldValue = AnwUtils::xmlDumpNodeChilds($oValueElement);
				$asValues[] = $mFieldValue;
		}
		$this->setContentFieldValues($sFieldName, $asValues);
	}
	
	//warning, it returns doc and not documentElement!
	function toXml($bWithDefaultValues=true)
	{
		if (true || !isset($this->oCachedXml[$bWithDefaultValues]))
		{
			$oDoc = AnwUtils::newDomDocument();
			$oNodeRoot = $oDoc->createElement('doc');
			$oDoc->appendChild($oNodeRoot);
			
			//export fields
			$aoFields = $this->getContentFieldsContainer()->getContentFields();
			foreach ($aoFields as $sFieldName => $oContentField)
			{
				$oNodeField = $oDoc->createElement($sFieldName);
				$asFieldValues = $this->getContentFieldValues($sFieldName, false, $bWithDefaultValues);
				
				//export field values
				//TODO : use getContentFieldValuesAsXml ?
				
				foreach ($asFieldValues as $sFieldValue) //!!Use foreach instead of while which skips empty strings
				{
					$oValueNode = AnwXml::xmlCreateElementWithChilds($oNodeRoot->ownerDocument, self::XMLNODE_VALUE, $sFieldValue);
					$oNodeField->appendChild($oValueNode);
				}
				
				$oNodeRoot->appendChild($oNodeField);
			}
		
			$this->oCachedXml[$bWithDefaultValues] = $oDoc;
		}
		
		return clone $this->oCachedXml[$bWithDefaultValues];
	}
	
	function toXmlString($bWithDefaultValues=true)
	{
		if (true || !isset($this->sCachedXmlString[$bWithDefaultValues]))
		{
			$oDoc = $this->toXml($bWithDefaultValues);
			$this->sCachedXmlString[$bWithDefaultValues] = AnwUtils::xmlDumpNodeChilds($oDoc->documentElement);
		}
		return $this->sCachedXmlString[$bWithDefaultValues];
	}
	
	protected function loadContentFieldsValuesFromXml($sXmlValue)
	{//print "<br/>".$this->getContentFieldsContainer()->getName()."::".htmlentities($sXmlValue)."<br/>";
		$oNodeRoot = AnwUtils::loadXML( '<doc>'.$sXmlValue.'</doc>' );
		
		$aoContentFields = $this->getContentFieldsContainer()->getContentFields();		
		foreach ($aoContentFields as $sFieldName => $oContentField)
		{
			//search <fieldname> tag in XML
			$aoFieldNameElements = AnwXml::xmlGetChildsByTagName($sFieldName, $oNodeRoot);
			//print "FIELDNAME:".$sFieldName."<br/>";
			//print htmlentities(AnwUtils::xmlDumpNode($oNodeRoot));
			//print_r($aoFieldNameElements);
			if (count($aoFieldNameElements)>0)
			{
				$asValues = array();
				//we shouldn't get more than one <fieldname> tag
				if (count($aoFieldNameElements)>1)
				{
					throw new AnwUnexpectedException("Found more than 1 tag for ".$sFieldName);
				}
				$oFieldNameElement = array_pop($aoFieldNameElements);
				//print_r($oFieldNameElement);
				//print htmlentities(AnwUtils::xmlDumpNodeChilds($oFieldNameElement));
				//loop for each <anwvalue> tag
				$aoValueElements = AnwXml::xmlGetChildsByTagName(self::XMLNODE_VALUE, $oFieldNameElement);
				foreach ($aoValueElements as $oValueElement)
				{
					$mFieldValue = AnwUtils::xmlDumpNodeChilds($oValueElement);
					//print "VALUE:".htmlentities($mFieldValue)."<br/>";
					$asValues[] = $mFieldValue;
				}
				$this->aasContentFieldsValues[$sFieldName] = $asValues;
			}
			else
			{
				unset($this->aasContentFieldsValues[$sFieldName]);
			}
		}
		$this->setXmlChanged();
	}
	
	function getContentFieldsContainer()
	{
		return $this->oContentFieldsContainer;
	}

	function pubcall($sArg)
	{
		return $this->getContentFieldsContainer()->pubcall($sArg, $this);
	}
	
	private function setXmlChanged()
	{
		$this->oCachedXml = null;
		$this->sCachedXmlString = null;
	}
	
	
	function getContentFieldsTabs()
	{
		$aoFieldsTabs = array();
		
		$aoFields = $this->getContentFieldsContainer()->getContentFields();
		foreach ($aoFields as $oField)
		{
			if ($oField instanceof AnwStructuredContentField_tab)
			{
				if ($oField->isDisplayed())
				{
					$aoFieldsTabs[] = $oField;
				}
			}
		}
		return $aoFieldsTabs;
	}
	
	
	/*
	 * Content edition throught HTML form.
	 */
	
	function renderEditHtmlForm($bFromPost, $sFormUrl="", $sSuffix="", $oContentOriginal=null)
	{
		$sHtmlEditForm = "";
		
		//special process for the first call
		if ($oContentOriginal == null)
		{
			$oContentOriginal = $this;
			$oContentOriginal->bHtmlEditFormHasErroneousFields = false;
			$oContentOriginal->bHtmlEditFormHasPhpCode = false;
			$oContentOriginal->bHtmlEditFormHasJsCode = false;
			$oContentOriginal->nHtmlEditFormCountErroneousChild = 0;
			
			$sFormUrl = str_replace('&amp;', '&', $sFormUrl);
			AnwAction::headJs('var g_editcontentform_url="'.$sFormUrl.'";');
			AnwAction::headJsSrc( AnwComponent::getGlobalUrlStaticDefault()."editcontent.js" );
			AnwAction::headCssSrc( AnwComponent::getGlobalUrlStaticDefault()."editcontent.css" );
			
			//render tabs if any
			$aoFieldsTabs = $this->getContentFieldsTabs();
			if (count($aoFieldsTabs)>0)
			{
				$sHtmlTabs = "";
				foreach ($aoFieldsTabs as $oFieldTab)
				{
					$sHtmlTabs .= $oFieldTab->renderEditTab($sSuffix);
				}
				$sHtmlEditForm .= <<<EOF

<div class="contentfield_tabs">
	$sHtmlTabs
	<div class="break;"></div>
</div>
EOF;
			}
		}
		
		$aoContentFields = $this->getContentFieldsContainer()->getContentFields();
		foreach ($aoContentFields as $oContentField)
		{
			$nPreviousCountErroneousChild = $oContentOriginal->nHtmlEditFormCountErroneousChild;
			
			$sFieldName = $oContentField->getName();
			
			$asValuesForRender = array();
			$asFieldValues = array();
			
			//do not load from post when contentfield is hidden
			if ($bFromPost && !$oContentField->isDisplayed())
			{
				$bReallyFromPost = false;
			}
			else
			{
				$bReallyFromPost = $bFromPost;
			}
			
			/*****************************************************************************
			 * CONTENTFIELD CONTAINER : recursively get value from it's subcontentfields
			 *****************************************************************************/
			
			if ($oContentField instanceof AnwStructuredContentField_composed)
			{
				if (!$bReallyFromPost)
				{
					// shared code for multiplicity single or multiple
					try
					{
						$aoSubContents = $this->getSubContents($sFieldName);
					}
					catch(AnwUnexpectedException $e)
					{
						//contentfield_container don't exist already. create it.
						$aoSubContents = array($this->newContent($oContentField));
					}
				}
				else
				{
					$aoSubContents = array();
					if ($oContentField->isMultiple())
					{
						//create as many empty subcontents as instances edited from post
						$asFieldsIdsFromPost = AnwEnv::_POST($oContentField->getInputName($sSuffix), array());
						
						foreach ($asFieldsIdsFromPost as $sFieldIdFromPost)
						{
							$aoSubContents[$sFieldIdFromPost] = $this->newContent($oContentField);
						}
					}
					else
					{
						//only 1 empty instance
						$aoSubContents[] = $this->newContent($oContentField);
					}
				}
						
				foreach ($aoSubContents as $sIndice => $oSubContent)
				{
					//update suffix
					if (!$bReallyFromPost)
					{
						$sSuffixId = AnwUtils::genUniqueIdNumeric();
					}
					else
					{
						$sSuffixId = $sIndice;
					}
					$sNewSuffix = $oContentField->updateSuffix($sSuffix, $sSuffixId);
					
					$sSubRender = $oSubContent->renderEditHtmlForm($bReallyFromPost, $sFormUrl, $sNewSuffix, $oContentOriginal); //recursive call
					
					//$asValuesForRender[$sSuffixId] = $sSubRender;
					$asValuesForRender[$sSuffixId] = array(self::IDX_SUBCONTENT=>$oSubContent, self::IDX_RENDERED=>$sSubRender);
					
					if ($oContentField->isCollapsed() || $oContentField->isCollapsedChild())
					{
						$bWasCollapsingEnabled = AnwStructuredContentField::getDoCollapsing();
						AnwStructuredContentField::setDoCollapsing(false);
						
						$sSubRenderUncollapsed = $oSubContent->renderEditHtmlForm($bReallyFromPost, $sFormUrl, $sNewSuffix, $oContentOriginal); //recursive call
						$asValuesForRender[$sSuffixId]['UNCOLLAPSED'] = $sSubRenderUncollapsed;
						
						AnwStructuredContentField::setDoCollapsing($bWasCollapsingEnabled);
					}
					
					//only after calling render on subContent!
					$asFieldValues[] = $oSubContent->toXmlString();
				}
			}
			
				
			/*****************************************************************************
			 * CONTENTFIELD WITH DATA : get its value
			 *****************************************************************************/
			 
			else
			{
				if (!$bReallyFromPost)
				{
					$asFieldValues = $this->getContentFieldValues($sFieldName);
				}
				else
				{
					$asFieldValues = $oContentField->getValuesFromPost($sSuffix);
				}
				$asValuesForRender = $asFieldValues; 
			}
			
			$sFieldError = false;
			
			// - run the test for any _atomic field.
			// - only run the test for _composed fields which dont't have erroneous childs (important for security reasons, to not trigger test() procedure of these composed fields on unsafe values)
			if ($oContentOriginal->nHtmlEditFormCountErroneousChild==$nPreviousCountErroneousChild)
			{
				//test values and multiplicity (even for composed fields)
				try{
					$oContentField->testContentFieldValues($asFieldValues);
				}
				//catch(AnwInvalidContentException $e){
				catch(AnwException $e){ //here we can get errors from php edition
					//display error
					$sInputName = $oContentField->getInputName($sSuffix);
					$sFieldError = str_replace('#CONTENTFIELDINPUTID#', $sInputName, $e->getMessage());
					$oContentOriginal->bHtmlEditFormHasErroneousFields = true;
					$oContentOriginal->nHtmlEditFormCountErroneousChild++;
					
					//quick hack to solve the following problem:
					//if user enters invalid XML code, in a contentfield such as _xml
					//the system would fail on setContentFieldValues() or later, 
					//when trying to load invalid XML from the content.
					//
					//so we replace erroneous values by a valid-xml string.
					//these erroneous values should never be read later :
					// - test for this contentfield was done before
					// - render is done on $asValuesForRender, so that user views erroneous in edit inputs
					// - if this contentfield is child of a container, the container won't do the test as it contains already erroneous values
					if ($e instanceof AnwInvalidContentFieldValueException 
						&& $oContentField instanceof AnwStructuredContentField_atomic)
					{
						$sErroneousValue = '#ERRONEOUSVALUE#'; 
						foreach ($asFieldValues as $nValueIndice => $null)
						{
							$asFieldValues[$nValueIndice] = $sErroneousValue;
						}
					}
				}/*
				catch(AnwException $e){
					//should never go here
					print 'ERROR 62';
					print_r($e); exit;
				}*/
			}
			/*else
			{
				$sFieldError = "(test skipped)";
			}*/
			
			//render it now
			if ($oContentField->isDisplayed())
			{
				$sHtmlEditForm .= $oContentField->renderEditInputs($asValuesForRender, $sSuffix, $sFieldError);
			}
			
			if ($bReallyFromPost)
			{
				//update content for preview/save
				$this->setContentFieldValues($sFieldName, $asFieldValues);
			}
			
			//check acls - are these checks still needed, as test was integrated in class_contentfield?
			foreach ($asFieldValues as $mContentFieldValue)
			{
				//check PHP permission
				$bHasPhpCode = AnwUtils::contentHasPhpCode($mContentFieldValue);
				if ($bHasPhpCode)
				{
					$oContentOriginal->bHtmlEditFormHasPhpCode = true;
					AnwCurrentSession::getUser()->checkPhpEditionAllowed();
				}
				
				//check JS permission
				$bHasJsCode = AnwUtils::contentHasJsCode($mContentFieldValue);
				if ($bHasJsCode)
				{
					$oContentOriginal->bHtmlEditFormHasJsCode = true;
					AnwCurrentSession::getUser()->checkJsEditionAllowed();
				}
			}
		}		
		return $sHtmlEditForm;
	}
	
	function htmlEditFormHasErroneousFields()
	{
		return $this->bHtmlEditFormHasErroneousFields;
	}
	
	function htmlEditFormHasPhpCode()
	{
		return $this->bHtmlEditFormHasPhpCode;
	}
	
	function htmlEditFormHasJsCode()
	{
		return $this->bHtmlEditFormHasJsCode;
	}
}

class AnwContentPage extends AnwStructuredContent
{
	private $tmp_nTranslatedValuesTranslated;
	private $tmp_nTranslatedValuesTotal;
	
	function __construct($oContentFieldsContainer)
	{
		parent::__construct($oContentFieldsContainer);
	}
	
	function newContent($oContentFieldsContainer)
	{
		return new AnwContentPage($oContentFieldsContainer);
	}
	
	static function rebuildContentFromXml($oContentFieldsContainer, $sXmlValue)
	{
		$oContent = new AnwContentPage($oContentFieldsContainer);
		$oContent->loadContentFieldsValuesFromXml($sXmlValue);
		return $oContent;
	}
	
	private function prepareForOutput($oPage)
	{
		AnwPlugins::hook("content_prepare_for_output", $this, $oPage);
	}
	
	/**
	 * Used for content execution preview (by action edit).
	 */
	function toHtml($oPage)
	{
		$this->prepareForOutput($oPage);
		return $this->getContentFieldsContainer()->toHtml($this, $oPage);
	}
	
	function toFeedItem($oPage)
	{
		$this->prepareForOutput($oPage);
		return $this->getContentFieldsContainer()->toFeedItem($this, $oPage);
	}
	
	function setUntranslated()
	{
		$aoContentFields = $this->getContentFieldsContainer()->getContentFields();
		foreach ($aoContentFields as $sFieldName => $oContentField)
		{
			if ($oContentField->isTranslatable())
			{
				$asValues = array();
				if ($oContentField instanceof AnwStructuredContentField_composed)
				{
					$aoSubContents = $this->getSubContents($sFieldName);
					foreach ($aoSubContents as $i => $oSubContent)
					{
						$oSubContent->setUntranslated();
						$asValues[] = AnwUtils::xmlDumpNodeChilds($oSubContent->toXml()->documentElement);
					} 
				}
				else
				{
					$asValues = $this->getContentFieldValues($sFieldName);
					foreach ($asValues as $i => $sValue)
					{
						$oValueXml = AnwUtils::loadXML('<doc>'.$sValue.'</doc>');
						$oValueXml = AnwXml::xmlAddTranslateTags($oValueXml); //contentfield already checked
						$asValues[$i] = AnwUtils::xmlDumpNodeChilds($oValueXml);
					}
				}
				$this->setContentFieldValues($sFieldName, $asValues);
			}
		}
	}
	
	function isTranslated()
	{
		foreach ($this->aasContentFieldsValues as $sFieldName => $asContentFieldsValues)
		{
			$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
			if ($oContentField->isTranslatable())
			{
				$sTmp = implode(' ', $asContentFieldsValues);
				$bTranslated = (! preg_match(AnwUtils::getRegexpUntr(), $sTmp));
				if (!$bTranslated) return false;
			}
		}
		return true;		
	}
	
	function getTranslatedPercent()
	{
		$this->tmp_nTranslatedValuesTranslated = 0;
		$this->tmp_nTranslatedValuesTotal = 0;
		
		$fOnValue = "getTranslatedPercent_onContentFieldValue";
		AnwUtils::runCallbacksOnTranslatableField($this, $this, $fOnValue);
		
		$nPercent = $this->tmp_nTranslatedValuesTotal>0 ? ceil(($this->tmp_nTranslatedValuesTranslated/$this->tmp_nTranslatedValuesTotal)*100) : 100;
		
		//AnwDebug::logdetail("getTranslatedPercent result : translated=".$this->tmp_nTranslatedValuesTranslated.", total=".$this->tmp_nTranslatedValuesTotal.", percent=".$nPercent."%");
		return $nPercent;
	}
	
	function getTranslatedPercent_onContentFieldValue($oContentField, $oXmlValue, $sInputName)
	{
		$fOnTextValue = "getTranslatedPercent_onTextValue";
		AnwUtils::runCallbacksOnTranslatableValue($this, $oXmlValue, $sInputName, $fOnTextValue);
	}
	
	function getTranslatedPercent_onTextValue($oRootNode, $sInputName)
	{
		$this->tmp_nTranslatedValuesTotal++;
		if (!AnwXml::xmlIsUntranslated($oRootNode))
		{
			$this->tmp_nTranslatedValuesTranslated++;
		}
	}
	
	function getContentFieldIndexedValues($sFieldName)
	{
		$asContentFieldValues = $this->getContentFieldValues($sFieldName, true);
		$oContentField = $this->getContentFieldsContainer()->getContentField($sFieldName);
		foreach ($asContentFieldValues as $i=>$null)
		{
			$sIndexedValue = $asContentFieldValues[$i];
			$sIndexedValue = $oContentField->getIndexedValue($sIndexedValue);
			$sIndexedValue = substr($sIndexedValue, 0, AnwUtils::MAXLEN_INDEXVALUE);
		}
		return $asContentFieldValues;
	}
}


class AnwContentSettings extends AnwStructuredContent
{
	function __construct($oContentFieldsContainer)
	{
		parent::__construct($oContentFieldsContainer);
	}
	
	function newContent($oContentFieldsContainer)
	{
		return new AnwContentSettings($oContentFieldsContainer);
	}
	
	static function rebuildContentFromXml($oContentFieldsContainer, $sXmlValue)
	{
		$oContent = new AnwContentSettings($oContentFieldsContainer);
		$oContent->loadContentFieldsValuesFromXml($sXmlValue);
		return $oContent;
	}
	
	function getComponent()
	{
		return $this->getContentFieldsContainer()->getComponent();
	}
	
	/*
	 * I/O operations for settings files.
	 */
	
	function readSettings()
	{
		$sFileDefault = $this->getComponent()->getConfigurableFileDefault();
		if (file_exists($sFileDefault))
		{
			//read override
			try {
				$sFileOverride = $this->getComponent()->getConfigurableFileOverride();
				if (file_exists($sFileOverride)) {
					$cfg = null; //defined in file
					(require ($sFileOverride)) or die("Unable to load configuration file : ".$sFileOverride);
					$this->doReadSettings($cfg, false);
				}
			}
			catch(AnwException $e){}
			
			//read default settings
			try
			{
				$cfg = null; //defined in file
				(require ($sFileDefault)) or die("Unable to load configuration file : ".$sFileDefault); //no require_once here!
				$this->doReadSettings($cfg, true);
			}
			catch(AnwException $e){}
			
			//print "===================================<br/>";
			//print htmlentities($this->toXmlString(false));exit;
		}
	}
	
	private function doReadSettings($cfg, $bIsDefaultValue=false)
	{
		$aoConfigurableSettings = $this->getContentFieldsContainer()->getContentFields();
		
		foreach ($aoConfigurableSettings as $oSetting)
		{
			$sSettingName = $oSetting->getName();
			//print_r($this->getContentFieldValues($sSettingName));
			if (isset($cfg[$sSettingName]) /*&& count($this->getContentFieldValues($sSettingName))==0*/)
			{
				if ($oSetting instanceof AnwStructuredContentField_atomic)
				{
					$aoValues = array();
					
					//get values
					if ($oSetting->isMultiple())
					{
						foreach ($cfg[$sSettingName] as $mConfigItem)
						{
							$oValue = $oSetting->getValueFromConfigItem($mConfigItem);
							$aoValues[] = $oValue;
						}
					}
					else
					{
						$oValue = $oSetting->getValueFromConfigItem($cfg[$sSettingName]);
						$aoValues[] = $oValue;
					}
					/*if (count($aoValues)>0)
					{*/
						// initialize default settings values from default config file
						// set settings from the override file
						if ($bIsDefaultValue)
						{
							if ($oSetting->isMultiple())
							{
								$oSetting->setDefaultValues($aoValues);
							}
							else
							{
								if (count($aoValues)==1)
								{
									$oSetting->setDefaultValue($aoValues[0]);
								}
								else
								{
									//should never go here
									//print "no default for ".$sSettingName."**<br/>";
								}
							}
						}
						else
						{
							$this->setContentFieldValues($sSettingName, $aoValues);
						}
					/*}
					else
					{
						throw new AnwUnexpectedException("No value found from config");
					}*/
				}
				else
				{
					//recursive call
					
					if ($oSetting->isMultiple())
					{
						$aaSubCfg = $cfg[$sSettingName];
					}
					else
					{
						$aaSubCfg = array($cfg[$sSettingName]);
					}
					
					//we don't want to load default values here
					$aoSubContents = $this->getSubContents($sSettingName, false);
					
					$asSubValues = array();
					foreach ($aaSubCfg as $amSubCfg)
					{
						$i = count($asSubValues);
						if (isset($aoSubContents[$i]))
						{
							$oSubContent = $aoSubContents[$i];
						}
						else
						{
							$oSubContent = new AnwContentSettings($oSetting);
						}
						try
						{
							$oSubContent->doReadSettings($amSubCfg, $bIsDefaultValue);
							//very important, we dont want default values here!
							//so that we are sure that each set value comes from override
							if (!$bIsDefaultValue)
							{
								$asSubValues[] = $oSubContent->toXmlString(false);
							}
							else
							{
								$asSubValues[] = $oSubContent->toXmlString(true);
							}							
						}
						catch(AnwUnexpectedException $e){
							//no value found from config, skip it.
							//even if no value was found, it has maybe initialized some default values
						}
					}
					if (count($asSubValues)>0)
					{
						if (!$bIsDefaultValue)
						{
							$this->setContentFieldValues($sSettingName, $asSubValues);
						}
						else
						{
							$oSetting->setDefaultValues($asSubValues);
						}
					}
				}
			}
		}
	}
	
	function writeSettingsOverride()
	{
		$cfg = $this->toCfgArray();
		$sPhpCode = '<?php '."\n";
		$sPhpCode .= ' /**'."\n";
		$sPhpCode .= '  * Anwiki override file.'."\n";
		$sPhpCode .= '  * This file can be edited directly from file system, or from Anwiki web interface.'."\n";
		$sPhpCode .= '  * '."\n";
		$sPhpCode .= '  * Overridden file: '.$this->getComponent()->getConfigurableFileDefault()."\n";
		$sPhpCode .= '  * Generated on: '.Anwi18n::datetime(time())."\n";
		$sPhpCode .= '  * By: '.AnwCurrentSession::getUser()->getLogin()."\n";
		$sPhpCode .= '  * Using version: '.ANWIKI_VERSION_NAME.' ('.ANWIKI_VERSION_ID.')'."\n";
		$sPhpCode .= '  */'."\n";
		$sPhpCode .= "\n";
		$sPhpCode .= '$cfg = '.AnwUtils::arrayToPhp($cfg)."\n";
		$sPhpCode .= '?>';
		
		$sFileOverride = $this->getComponent()->getConfigurableFileOverride();
		AnwUtils::file_put_contents($sFileOverride, $sPhpCode, LOCK_EX);
	}
	
	private function areSimilarCfgArrays($aoDefaultValues, $aoSettingValues, $bCheckIndices)
	{
		if ($bCheckIndices)
		{
			//check indices
			foreach ($aoDefaultValues as $i => $oDefaultValue)
			{
				if (!isset($aoSettingValues[$i]) || $aoSettingValues[$i]!=$oDefaultValue)
				{
					return true;
				}
			}
			foreach ($aoSettingValues as $i => $oSettingValue)
			{
				if (!isset($aoDefaultValues[$i]) || $aoDefaultValues[$i]!=$oSettingValue)
				{
					return true;
				}
			}
		}
		else
		{
			//only check values
			foreach ($aoDefaultValues as $i => $oDefaultValue)
			{
				if (!in_array($oDefaultValue, $aoSettingValues))
				{
					return true;
				}
			}
			foreach ($aoSettingValues as $i => $oSettingValue)
			{
				if (!in_array($oSettingValue, $aoDefaultValues))
				{
					return true;
				}
			}
		}
		return false;
	}
	
	private function toCfgArray()
	{
		$cfg = array();
		$aoConfigurableSettings = $this->getContentFieldsContainer()->getContentFields();
		foreach ($aoConfigurableSettings as $oSetting)
		{
			$sSettingName = $oSetting->getName();
			
			if ($oSetting instanceof AnwStructuredContentField_atomic)
			{
				$aoSettingValues = $this->getContentFieldValues($sSettingName);
				
				$bMultipleValuesAreDifferent = false;
				if ($oSetting->isMultiple())
				{
					try
					{
						$aoDefaultValues = $oSetting->getDefaultValues();
					}
					catch(AnwException $e){
						$aoDefaultValues = array();
					}
					//quick test
					$bCheckIndices = $oSetting->getMultiplicity()->isSortable();
					$bMultipleValuesAreDifferent = self::areSimilarCfgArrays($aoDefaultValues, $aoSettingValues, $bCheckIndices);
				}
				else
				{
					$aoDefaultValues = array($oSetting->getDefaultValue());
				}
				
				foreach ($aoSettingValues as $oSettingValue)
				{
					if ($bMultipleValuesAreDifferent || !$oSetting->hasDefaultValue() ||
						($oSetting->hasDefaultValue() && !in_array($oSettingValue, $aoDefaultValues)/*$oSettingValue!=$oSetting->getDefaultValue()*/)
					)
					{
						$cfg[$sSettingName][] = $oSetting->getConfigItemFromValue($oSettingValue);
					}
				}
			}
			else
			{
				//recursive call
				$aoSubContents = $this->getSubContents($sSettingName);
				foreach ($aoSubContents as $oSubContent)
				{
					$amSubCfg = $oSubContent->toCfgArray();
					if (count($amSubCfg)>0)
					{
						$cfg[$sSettingName][] = $amSubCfg;
					}
				}
			}
			
			if (!$oSetting->isMultiple() && isset($cfg[$sSettingName]))
			{
				$cfg[$sSettingName] = array_pop($cfg[$sSettingName]);
			}
		}
		return $cfg;
	}
}

abstract class AnwStructuredContentEditionForm
{
	private $oContent;
	private $sFormUrl;
	private $sRender;
	
	const POSTED_INPUT_NAME = "editionformposted";
	
	function __construct($oContent, $sFormUrl)
	{
		$this->oContent = clone $oContent;
		$this->sFormUrl = $sFormUrl;
	}
	
	function getRender($bForceFromPost=false)
	{
		if (!$this->sRender)
		{
			$bFromPost = ($bForceFromPost || $this->isPosted());
			$this->sRender = $this->oContent->renderEditHtmlForm($bFromPost, $this->sFormUrl);
			$this->sRender .= '<input type="hidden" name="'.self::POSTED_INPUT_NAME.'" value="1"/>';
		}
		return $this->sRender;
	}
	
	function hasErrors()
	{
		return $this->oContent->htmlEditFormHasErroneousFields();
	}
	
	function getContent()
	{
		return $this->oContent;
	}
	
	function isPosted()
	{
		return AnwEnv::_POST(self::POSTED_INPUT_NAME);
	}
}

class AnwStructuredContentEditionFormSettings extends AnwStructuredContentEditionForm
{
	private $oEditableComponent;
	
	function __construct($oEditableComponent, $oConfigurableContent, $sFormUrl)
	{
		parent::__construct($oConfigurableContent, $sFormUrl);
		
		$this->oEditableComponent = $oEditableComponent;
	}
	
	function getRender($bForceFromPost=false)
	{
		$sRender = parent::getRender($bForceFromPost);
		
		$sConfigFileOverride = $this->getEditableComponent()->getConfigurableFileOverride();
		$sRender .= '<div class="explain" style="float:right; width:65%">'.AnwComponent::g_editcontent("editsettings_info", array('filename'=>'<span style="font-size:0.8em">'.$sConfigFileOverride.'</span>')).'</div>';
		
		return $sRender;
	}
	
	function saveEdition()
	{
		try
		{
			//make sure file is writable
			$sConfigFileOverride = $this->getEditableComponent()->getConfigurableFileOverride();
			if (!file_exists($sConfigFileOverride))
			{
				try
				{
					AnwUtils::file_put_contents($sConfigFileOverride, "");
				}
				catch(AnwException $e){}
			}
			if (!is_writable($sConfigFileOverride))
			{
				$sError = AnwComponent::g_("err_need_write_file", array('filename'=>$sConfigFileOverride, 'processuser'=>AnwEnv::getProcessUser(), 'processgroup'=>AnwEnv::getProcessGroup()));
				throw new AnwStructuredContentEditionFormException($sError);
			}
			
			//update content from post
			$this->getRender(true);
			
			//check errors
			if ($this->hasErrors())
			{
				throw new AnwInvalidContentException();
			}
			
			//save
			$this->getContent()->writeSettingsOverride();		
		}
		catch (AnwInvalidContentException $e)
		{
			$sError = AnwComponent::g_("err_contentinvalid");
			throw new AnwStructuredContentEditionFormException($sError);
		}
		catch(AnwUnexpectedException $e)
		{
			$sError = AnwComponent::g_("err_ex_unexpected_p");
			$nErrorNumber = AnwDebug::reportError($e);
			if ($nErrorNumber)
			{
				$sError .= '<br/>'.$this->g_("err_ex_report",array("errornumber"=>$nErrorNumber));
			}
			throw new AnwStructuredContentEditionFormException($sError);
		}
	}
	
	function getEditableComponent()
	{
		return $this->oEditableComponent;
	}
	
	
}



class AnwStructuredContentEditionFormPage extends AnwStructuredContentEditionForm
{
	private $oEditableComponent;
	
	function __construct($oContentPage, $sFormUrl)
	{
		parent::__construct($oContentPage, $sFormUrl);
	}
	
	function updateContentFromEdition()
	{
		try
		{
			//update content from post
			$this->getRender(true);
			
			//check errors
			if ($this->hasErrors())
			{
				throw new AnwInvalidContentException();
			}
			
			return $this->getContent();
		}
		catch (AnwInvalidContentException $e)
		{
			$sError = AnwComponent::g_("err_contentinvalid");
			throw new AnwStructuredContentEditionFormException($sError);
		}
		catch (AnwAclPhpEditionException $e)
		{
			$sError = $e->getMessage();
			throw new AnwStructuredContentEditionFormException($sError);
		}
		catch(AnwUnexpectedException $e)
		{
			$sError = AnwComponent::g_("err_ex_unexpected_p");
			$nErrorNumber = AnwDebug::reportError($e);
			if ($nErrorNumber)
			{
				$sError .= '<br/>'.$this->g_("err_ex_report",array("errornumber"=>$nErrorNumber));
			}
			throw new AnwStructuredContentEditionFormException($sError);
		}
	}
}

?>